package com.gx.obe.step.composite;

import java.text.MessageFormat;

import org.eclipse.swt.SWT;
import org.eclipse.swt.core.MessageDialog;
import org.eclipse.swt.core.utils.WidgetUtils;
import org.eclipse.swt.core.widgets.ESepator;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Composite;

import com.gx.obe.ColorConstants;
import com.gx.obe.common.widget.button.EvalLeftButton;
import com.gx.obe.common.widget.button.EvaluationButton;
import com.gx.obe.components.core.Constants;
import com.gx.obe.components.core.enumeration.CommonEnum;
import com.gx.obe.message.Messages;
import com.gx.obe.step.Activator;
import com.gx.obe.step.listener.ExpertEvaluationListener;
import com.gx.obe.step.service.StepService;
import com.gx.obe.util.utils.StringUtils;
import com.gx.obe.web.entity.auth.AuthUser;
import com.gx.obe.web.entity.evaluation.EvaluationStep;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.swtdesigner.ResourceManager;

/**
 * @Description: 评审界面
 * @author chenxw
 */
public abstract class EvaluationComposite extends Composite {
	
	/**
	 * @Description: 步骤服务
	 */
	private StepService stepService = new StepService();
	
	/**
	 * @Description: 专家评审接听
	 */
	private ExpertEvaluationListener expertEvaluationListener;
	
	/**
	 * @Description: 左侧工具栏
	 */
	private Composite left_tool_composite;
	
	/**
	 * @Description: 评审区域
	 */
	private Composite evaluatoinComposite;
	/**
	 * @Description: 按钮区域
	 */
	private Composite buttonComposite;
	
	/**
	 * @Description: 当前项目
	 */
	protected TenderProjectEntity tenderProject;
	/**
	 * @Description: 当前评审步骤
	 */
	protected EvaluationStep evaluationStep;
	/**
	 * @Description: 当前登陆用户
	 */
	protected AuthUser curUser = Constants.USER;
	
	/**
	 * @author chenxw
	 * @param parent
	 * @param style
	 */
	public EvaluationComposite(Composite parent, int style) {
		super(parent, SWT.NONE);
		
		GridLayout gridLayout = new GridLayout(2, false);
		gridLayout.verticalSpacing = 0;
		gridLayout.horizontalSpacing = 0;
		gridLayout.marginHeight = 0;
		gridLayout.marginWidth = 0;
		setLayout(gridLayout);
		
		left_tool_composite = new Composite(this, SWT.NONE);
		GridLayout gl_left_tool_composite = new GridLayout(1, false);
		left_tool_composite.setLayout(gl_left_tool_composite);
		left_tool_composite.setBackground(EvalLeftButton.BG);
		left_tool_composite.setBackgroundMode(SWT.INHERIT_FORCE);
		left_tool_composite.setLayoutData(new GridData(SWT.LEFT, SWT.FILL, false, true, 1, 1));
		
		EvalLeftButton goto_back = new EvalLeftButton(left_tool_composite, SWT.SELECTED);
		goto_back.addMouseListener(MouseListener.mouseUpAdapter(e -> goBackAction()));
		goto_back.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, false, false, 1, 1));
		goto_back.setImage(ResourceManager.getPluginImage(Activator.PLUGIN_ID, "icons/back.png"));
		goto_back.setToolTipText(Messages.Button_return);
		
		EvalLeftButton refresh = new EvalLeftButton(left_tool_composite, SWT.SELECTED);
		refresh.addMouseListener(MouseListener.mouseUpAdapter(e -> refreshEvaluationResult()));
		refresh.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, false, false, 1, 1));
		refresh.setImage(ResourceManager.getPluginImage(Activator.PLUGIN_ID, "icons/refresh.png"));
		refresh.setToolTipText(Messages.Button_refresh);
		
		Composite composite_1 = new Composite(this, SWT.NONE);
		composite_1.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		GridLayout gl_container = new GridLayout(1, false);
		gl_container.verticalSpacing = 0;
		gl_container.marginHeight = 0;
		gl_container.marginWidth = 0;
		gl_container.horizontalSpacing = 0;
		composite_1.setLayout(gl_container);
		
		evaluatoinComposite = new Composite(composite_1, SWT.NONE);
		evaluatoinComposite.setLayout(new FillLayout(SWT.HORIZONTAL));
		evaluatoinComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));

		createEvaluatoinComposite(evaluatoinComposite);
		
		ESepator sepLab = new ESepator(composite_1, SWT.HORIZONTAL);
		sepLab.setBackground(ColorConstants.SEP_LIGHT_COLOR);
		sepLab.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		
		buttonComposite = new Composite(composite_1, SWT.NONE);
		buttonComposite.setLayout(new RowLayout(SWT.HORIZONTAL));
		buttonComposite.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, true, false, 1, 1));
		buttonComposite.setBackgroundMode(SWT.INHERIT_FORCE);
		
		initButtonComposite();
		
	}
	
	/**
	 * @Description: 初始化按钮区域
	 * @author chenxw 
	 */
	protected void initButtonComposite() {
		createEButton(Messages.Button_saveEvaluationResult, MouseListener.mouseUpAdapter(e -> saveAction()));
		createEButton(Messages.Button_submitEvaluationResult, MouseListener.mouseUpAdapter(e -> submitAction()));
	}

	/**
	 * @Description: 创建按钮
	 * @author chenxw
	 * @param name
	 * @param listener
	 * @return 
	 */
	protected EvaluationButton createEButton(String name, MouseListener listener) {
		EvaluationButton button = new EvaluationButton(buttonComposite, SWT.NONE);
		button.setText(name);
		button.addMouseListener(listener);
		return button;
	}
	
	/**
	 * @Description: 创建评审界面
	 * @author chenxw
	 * @param fromComposite
	 */
	protected abstract void createEvaluatoinComposite(Composite evaluatoinComposite);
	
	/**
	 * @Description: 刷新评审结果
	 * @author chenxw
	 */
	protected abstract void refreshEvaluationResult();
	
	/**
	 * @Description: 保存评审结果
	 * @author chenxw
	 * @return
	 */
	protected abstract boolean saveEvaluationResult();
	
	/**
	 * @Description: 检查评审结果
	 * @author chenxw
	 * @return
	 */
	protected abstract String checkEvaluationResult();
	
	/**
	 * @Description: 刷新
	 * @author chenxw
	 * @param evaluationStep
	 */
	public void refresh(TenderProjectEntity tenderProject, EvaluationStep evaluationStep) {
		this.tenderProject = tenderProject;
		this.evaluationStep = evaluationStep;
		refreshEvaluationStatus();
		refreshEvaluationResult();
	}
	
	/**
	 * @Description: 刷新评审状态为进行中
	 * @author chenxw 
	 */
	private void refreshEvaluationStatus() {
		stepService.updateExpertEvaluationStatus(CommonEnum.IN_PROGRESS, tenderProject.getId(), evaluationStep, curUser.getId());
	}
	
	/**
	 * @Description: 返回事件
	 * @author chenxw
	 */
	private void goBackAction() {
		if (null != expertEvaluationListener) expertEvaluationListener.complete();
	}
	
	/**
	 * @Description: 保存事件
	 * @author chenxw
	 */
	private void saveAction() {
		if (!saveEvaluationResult()) {
			MessageDialog.openInformation(getShell(), Messages.Tips_saveEvalResultFailed);
			return;
		}
		MessageDialog.openShortMessage(Messages.Tips_saveEvalResultSuccess);
		refreshEvaluationResult();
	}
	
	/**
	 * @Description: 提交事件
	 * @author chenxw
	 */
	private void submitAction() {
		if (!saveEvaluationResult()) {
			MessageDialog.openInformation(getShell(), Messages.Tips_saveEvalResultFailed);
			return;
		}
		refreshEvaluationResult();
		String message = checkEvaluationResult();
		if (StringUtils.isNotEmpty(message)) {
			MessageDialog.openInformation(getShell(), message);
			return;
		}
		message = MessageFormat.format(Messages.Tips_submitEvaluationResult, evaluationStep.getStepName());
		if (!MessageDialog.openConfirm(getShell(), Messages.Button_submitEvaluationResult, message, Messages.Button_confirmComplate, Messages.Button_continueEvaluation)) {
			return;
		}
		if (!StepService.submitEvaluationStep(tenderProject.getId(), evaluationStep.getId(), curUser.getId())) {
			MessageDialog.openInformation(getShell(), Messages.Tips_submitEvalResultFailed);
			return;
		}
		goBackAction();
	}
	
	/**
	 * @Description: 隐藏左侧
	 * @author chenxw
	 * @param hideLeft 
	 */
	public void hideLeft(boolean hideLeft) {
		WidgetUtils.setControlVisible(left_tool_composite, !hideLeft);
	}
	
	/**
	 * @Description: 添加专家评审监听
	 * @author chenxw
	 * @param expertEvaluationListener
	 */
	public void addExpertEvaluationListener(ExpertEvaluationListener expertEvaluationListener) {
		this.expertEvaluationListener = expertEvaluationListener;
	}
	
	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}
}
