package com.gx.obe.step.composite;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.function.Function;
import java.util.stream.Collectors;

import org.eclipse.swt.SWT;
import org.eclipse.swt.core.utils.WidgetUtils;
import org.eclipse.swt.core.widgets.ELink;
import org.eclipse.swt.core.widgets.ESepator;
import org.eclipse.swt.core.widgets.RectangleRadioButton;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import com.gx.obe.ColorConstants;
import com.gx.obe.business.entity.ExpertEvaluationStep;
import com.gx.obe.common.icons.IconsActivator;
import com.gx.obe.common.widget.utils.MenuFunctionUtils;
import com.gx.obe.components.core.Constants;
import com.gx.obe.components.core.enumeration.CommonEnum;
import com.gx.obe.components.core.enumeration.ExpertEnum;
import com.gx.obe.message.Messages;
import com.gx.obe.step.dialog.ApplyReEvaluationDialog;
import com.gx.obe.step.service.StepService;
import com.gx.obe.util.utils.DateUtils;
import com.gx.obe.web.entity.auth.AuthUser;
import com.gx.obe.web.entity.evaluation.EvaluationStep;
import com.gx.obe.web.entity.expert.Expert;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.evaluation.EvaluationStepService;
import com.swtdesigner.ResourceManager;
import com.swtdesigner.SWTResourceManager;

/** 
 * @Description: 评审结果界面
 * @author chenxw
 * @param <E> 
 */
public abstract class EvaluationResultComposite<E> extends Composite {
	
	private EvaluationStepService evaluationStepService = new EvaluationStepService();
	
	private MenuFunctionUtils menuFunctionUtils = new MenuFunctionUtils();
	
	private StepService stepService = new StepService();
	
	private ELink applyReEvaluation_link;
	
	protected TenderProjectEntity tenderProject;
	protected EvaluationStep evaluationStep;
	protected AuthUser curUser = Constants.USER;
	protected ExpertEvaluationStep expertEvaluationStep;
	
	private Composite expert_list_composite;
	
	private RectangleRadioButton curExpertBtn;
	
	
	private Label message_label;
	
	private Composite load_composite;
	
	private boolean showApplyReEvaluation;
	
	private Map<String, ExpertEvaluationStep> expertEvaluationStepMap;
	
	private List<RectangleRadioButton> otherRectangleRadioButtonList = new ArrayList<>();
	
	/** 
	 * @Description: 类型
	 * @author chenxw 
	 */
	public enum Type {
		EXPERT, OTHER;
	}
	
	/**
	 * @author chenxw
	 * @param parent
	 * @param style
	 */
	public EvaluationResultComposite(Composite parent, int style) {
		super(parent, style);
		
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.verticalSpacing = 0;
		gridLayout.horizontalSpacing = 0;
		gridLayout.marginHeight = 1;
		gridLayout.marginWidth = 1;
		setLayout(gridLayout);
		
		Composite composite = new Composite(this, SWT.NONE);
		GridLayout gl_composite = new GridLayout(1, false);
		gl_composite.marginTop = 5;
		gl_composite.verticalSpacing = 0;
		gl_composite.marginWidth = 0;
		gl_composite.marginHeight = 0;
		composite.setLayout(gl_composite);
		composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
		
		expert_list_composite = new Composite(composite, SWT.NONE);
		RowLayout rl_expert_list_composite = new RowLayout(SWT.HORIZONTAL);
		rl_expert_list_composite.marginLeft = 0;
		rl_expert_list_composite.marginBottom = 0;
		rl_expert_list_composite.spacing = 5;
		expert_list_composite.setLayout(rl_expert_list_composite);
		expert_list_composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
		
		ESepator expert_tab = new ESepator(composite, SWT.NONE);
		expert_tab.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		expert_tab.setBackground(ColorConstants.SKIN_BG);
		expert_tab.setLineWidth(2);
		
		Composite message_composite = new Composite(this, SWT.NONE);
		GridLayout gl_composite_6 = new GridLayout(2, false);
		gl_composite_6.marginWidth = 0;
		message_composite.setLayout(gl_composite_6);
		message_composite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		message_label = new Label(message_composite, SWT.WRAP);
		message_label.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		message_label.setFont(SWTResourceManager.getFont("微软雅黑", 10, SWT.NORMAL));
		message_label.setForeground(ColorConstants.NOTE_TIPS_COLOR);
		
		Composite linkComposite = new Composite(message_composite, SWT.NONE);
		RowLayout rl_composite_1 = new RowLayout(SWT.HORIZONTAL);
		rl_composite_1.spacing = 5;
		rl_composite_1.marginBottom = 0;
		rl_composite_1.marginTop = 0;
		rl_composite_1.marginRight = 0;
		rl_composite_1.marginLeft = 0;
		linkComposite.setLayout(rl_composite_1);
		linkComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1));
		
		createLinkComposite(linkComposite);
		
		applyReEvaluation_link = new ELink(linkComposite, SWT.NONE);
		applyReEvaluation_link.addMouseListener(MouseListener.mouseUpAdapter(e -> applyReEvaluationAction()));
		applyReEvaluation_link.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/hand.png"));
		applyReEvaluation_link.setText(Messages.Button_appllyForBack);
		menuFunctionUtils.addFunction(applyReEvaluation_link, "APPLLY_FOR_BACK");
		
		Composite composite_2 = new Composite(this, SWT.NONE);
		GridLayout gl_composite_2 = new GridLayout(1, false);
		gl_composite_2.marginHeight = 1;
		gl_composite_2.marginWidth = 1;
		composite_2.setLayout(gl_composite_2);
		composite_2.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		composite_2.setBackground(ColorConstants.BORDER_LINE_COLOR);
		
		load_composite = new Composite(composite_2, SWT.NONE);
		load_composite.setBackgroundMode(SWT.INHERIT_FORCE);
		load_composite.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		GridLayout gl_load_composite = new GridLayout(1, false);
		gl_load_composite.verticalSpacing = 0;
		gl_load_composite.horizontalSpacing = 0;
		gl_load_composite.marginHeight = 0;
		gl_load_composite.marginWidth = 0;
		load_composite.setLayout(gl_load_composite);
		load_composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		
		createLoadComposite(load_composite);
	}
	
	/**
	 * @Description: 创建按钮区域
	 * @author chenxw
	 * @param linkComposite 
	 */
	protected abstract void createLinkComposite(Composite linkComposite);

	/**
	 * @Description: 创建加载区域
	 * @author chenxw
	 * @param load_composite 
	 */
	protected abstract void createLoadComposite(Composite load_composite);
	
	/**
	 * @Description: 获取评审用户列表
	 * @author chenxw
	 * @param tenderProject
	 * @param evaluationStep
	 * @return 
	 */
	protected abstract List<Expert> getEvaluationUserList(TenderProjectEntity tenderProject, EvaluationStep evaluationStep);
	
	/**
	 * @Description: 刷新评审结果
	 * @author chenxw
	 * @param expert 
	 */
	protected abstract void refreshEvaluationResult(Expert expert);
	
	/**
	 * @Description: 刷新评审结果
	 * @author chenxw
	 * @param e 
	 */
	protected abstract void refreshEvaluationResult(E e);
	
	/**
	 * @Description: 获取评审结果
	 * @author chenxw
	 * @param load_composite
	 * @param type 
	 */
	protected abstract void getEvaluationResult(Composite load_composite, Type type);
	
	/**
	 * @Description: 销毁评审结果界面
	 * @author chenxw
	 * @param type 
	 */
	protected abstract void disposeEvaluationResult(Type type);
	
	/**
	 * @Description: 评审结果界面是否销毁
	 * @author chenxw
	 * @param type
	 * @return 
	 */
	protected abstract boolean isDisposeEvaluationResult(Type type);
	
	/**
	 * @Description: 获取其他列表
	 * @author chenxw
	 * @return 
	 */
	protected abstract List<E> getOtherList();
	
	/**
	 * @Description: 获取其他按钮名称
	 * @author chenxw
	 * @param e
	 * @return 
	 */
	protected abstract String getOtherBtnName(E e);
	
	/**
	 * @Description: 刷新
	 * @author chenxw
	 * @param tenderProject
	 * @param evaluationStep 
	 */
	public void refresh(TenderProjectEntity tenderProject, EvaluationStep evaluationStep) {
		this.tenderProject = tenderProject;
		this.evaluationStep = evaluationStep;
		refreshExpertEvaluationStep();
		
		refreshExpertResult();
		refreshOtherResult();
		Arrays.stream(expert_list_composite.getChildren()).findFirst().ifPresent(t -> t.notifyListeners(SWT.MouseUp, null));
	}
	
	/**
	 * @Description: 刷新专家评审步骤
	 * @author chenxw 
	 */
	private void refreshExpertEvaluationStep() {
		applyReEvaluation_link.setText(MessageFormat.format(Messages.Button_appllyForBackFormat, evaluationStep.getStepName()));
		expertEvaluationStep = stepService.getExpertEvaluationStep(tenderProject.getId(), evaluationStep.getId(), curUser.getId());
		showApplyReEvaluation = Objects.nonNull(expertEvaluationStep) && CommonEnum.COMPLETE.equals(expertEvaluationStep.getStepStatus());
	}
	
	/**
	 * @Description: 刷新专家结果
	 * @author chenxw 
	 */
	private void refreshExpertResult() {
		expertEvaluationStepMap = stepService.getEvaluationStepListByStepId(tenderProject.getId(), evaluationStep.getId()).stream()
				.collect(Collectors.toMap(ExpertEvaluationStep::getUserId, Function.identity()));
		List<Expert> expertList = getEvaluationUserList(tenderProject, evaluationStep);
		if (Objects.isNull(expertList)) {
			return;
		}
		expertList.stream().forEach(this::createRectangleRadioButton);
	}
	
	/**
	 * @Description: 刷新其他结果
	 * @author chenxw 
	 */
	private void refreshOtherResult() {
		otherRectangleRadioButtonList.clear();
		List<E> list = getOtherList();
		if (Objects.isNull(list)) {
			return;
		}
		list.stream().forEach(this::createOtherRectangleRadioButton);
	}
	
	/**
	 * @Description: 创建专家按钮
	 * @author chenxw
	 * @param expert 
	 */
	private void createRectangleRadioButton(Expert expert) {
		RectangleRadioButton expertBtn = new RectangleRadioButton(expert_list_composite, SWT.NONE);
		String userName = expert.getUser().getUserName();
		if (ExpertEnum.LEADER_YES.equals(expert.getIsLeader())) {
			userName = userName.concat("（").concat(expert.getExpertTypeCN()).concat("）");
		}
		expertBtn.setText(userName);
		expertBtn.addMouseListener(MouseListener.mouseUpAdapter(e -> {
			Optional.ofNullable(curExpertBtn).ifPresent(t -> t.setSelection(false));
			curExpertBtn = expertBtn;
			curExpertBtn.setSelection(true);
			selectExpertBtn(expertBtn, expert);
		}));
		refreshExpertRectangleRadioButton(expertBtn, expertEvaluationStepMap.get(expert.getUserId()));
	}
	
	/**
	 * @Description: 创建其他按钮
	 * @author chenxw
	 * @param e 
	 */
	private void createOtherRectangleRadioButton(E e) {
		RectangleRadioButton otherBtn = new RectangleRadioButton(expert_list_composite, SWT.NONE);
		otherBtn.setText(getOtherBtnName(e));
		otherBtn.addMouseListener(MouseListener.mouseUpAdapter(me -> {
			Optional.ofNullable(curExpertBtn).ifPresent(t -> t.setSelection(false));
			curExpertBtn = otherBtn;
			curExpertBtn.setSelection(true);
			selectOtherBtn(e);
		}));
		otherRectangleRadioButtonList.add(otherBtn);
	}
	
	/**
	 * @Description: 刷新专家按钮图片
	 * @author chenxw
	 * @param expertBtn
	 * @param expertEvaluationStep 
	 */
	private void refreshExpertRectangleRadioButton(RectangleRadioButton expertBtn, ExpertEvaluationStep expertEvaluationStep) {
		Image image = Optional.ofNullable(expertEvaluationStep)
				.map(ExpertEvaluationStep::getStepStatus)
				.map(CommonEnum.COMPLETE::equals)
				.filter(t -> t)
				.map(t -> ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/green/ok.png"))
				.orElse(null);
		Optional.ofNullable(expertBtn).ifPresent(t -> t.setImage(image));
		refreshOtherRectangleRadioButtonList();
	}
	
	/**
	 * @Description: 刷新其他按钮列表图片
	 * @author chenxw 
	 */
	private void refreshOtherRectangleRadioButtonList() {
		evaluationStep = evaluationStepService.getById(evaluationStep.getId());
		Image image = null;
		if (CommonEnum.COMPLETE.equals(evaluationStep.getStepStatus())) {
			image = ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/green/ok.png");
		}
		for (RectangleRadioButton rectangleRadioButton : otherRectangleRadioButtonList) {
			rectangleRadioButton.setImage(image);
		}
	}
	
	/**
	 * @Description: 选择专家按钮
	 * @author chenxw
	 * @param expertBtn
	 * @param expert 
	 */
	private void selectExpertBtn(RectangleRadioButton expertBtn, Expert expert) {
		ExpertEvaluationStep step = null;
		boolean eqUserId = curUser.getId().equals(expert.getUserId());
		if (eqUserId) {
			step = expertEvaluationStep;
		} else {
			step = stepService.getExpertEvaluationStep(tenderProject.getId(), evaluationStep.getId(), expert.getUserId());
		}
		message_label.setText(getMessage(step));
		WidgetUtils.setControlVisible(applyReEvaluation_link, showApplyReEvaluation && eqUserId);
		if (isDisposeEvaluationResult(Type.EXPERT)) {
			disposeEvaluationResult(Type.OTHER);
			getEvaluationResult(load_composite, Type.EXPERT);
			load_composite.layout();
		}
		refreshEvaluationResult(expert);
		refreshExpertRectangleRadioButton(expertBtn, step);
	}
	
	/**
	 * @Description: 选择其他按钮
	 * @author chenxw
	 * @param e 
	 */
	private void selectOtherBtn(E e) {
		message_label.setText(getOtherMessage());
		WidgetUtils.setControlVisible(applyReEvaluation_link, false);
		if (isDisposeEvaluationResult(Type.OTHER)) {
			disposeEvaluationResult(Type.EXPERT);
			getEvaluationResult(load_composite, Type.OTHER);
			load_composite.layout();
		}
		refreshEvaluationResult(e);
	}
	
	/**
	 * @Description: 获取专家消息
	 * @author chenxw
	 * @param step
	 * @return 
	 */
	private String getMessage(ExpertEvaluationStep step) {
		if (Objects.isNull(step)) {
			return MessageFormat.format(Messages.Tips_progressNotStart, evaluationStep.getStepName());
		}
		StringBuilder msg = new StringBuilder();
		msg.append("[").append(evaluationStep.getStepName()).append("]");
		msg.append(Messages.EvalStartTime).append("：");
		msg.append(Optional.of(step.getStartTime()).map(DateUtils::getTimestampFormat).orElse("????-??-?? ??:??:??"));
		msg.append("---");
		if (CommonEnum.COMPLETE.equals(step.getStepStatus())) {
			msg.append(Messages.EvalEndTime).append("：");
			msg.append(Optional.of(step.getEndTime()).map(DateUtils::getTimestampFormat).orElse("????-??-?? ??:??:??"));
		} else {
			msg.append(MessageFormat.format(Messages.Tips_progressIn, evaluationStep.getStepName()));
		}
		return msg.toString();
	}
	
	/**
	 * @Description: 获取其他消息
	 * @author chenxw
	 * @return 
	 */
	private String getOtherMessage() {
		StringBuilder msg = new StringBuilder();
		if (CommonEnum.COMPLETE.equals(evaluationStep.getStepStatus())) {
			msg.append(MessageFormat.format(Messages.Tips_progressComplate, evaluationStep.getStepName()));
		} else if (CommonEnum.IN_PROGRESS.equals(evaluationStep.getStepStatus()) || CommonEnum.RE_OPERATOR.equals(evaluationStep.getStepStatus())) {
			msg.append(MessageFormat.format(Messages.Tips_progressIn, evaluationStep.getStepName()));
		} else {
			msg.append(MessageFormat.format(Messages.Tips_progressNotStart, evaluationStep.getStepName()));
		}
		return msg.toString();
	}
	
	/**
	 * @Description: 申请回退评审事件
	 * @author chenxw 
	 */
	protected void applyReEvaluationAction() {
		new ApplyReEvaluationDialog(getShell(), evaluationStep, expertEvaluationStep).open();
	}
	
	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}
	
}
