package com.gx.obe.step.composite;

import java.text.MessageFormat;
import java.util.Objects;

import org.eclipse.swt.SWT;
import org.eclipse.swt.core.utils.WidgetUtils;
import org.eclipse.swt.core.widgets.ELink;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import com.gx.obe.ColorConstants;
import com.gx.obe.business.entity.ExpertEvaluationStep;
import com.gx.obe.common.icons.IconsActivator;
import com.gx.obe.components.core.Constants;
import com.gx.obe.components.core.I18NEnum;
import com.gx.obe.components.core.enumeration.CommonEnum;
import com.gx.obe.message.Messages;
import com.gx.obe.step.dialog.ApplyReEvaluationDialog;
import com.gx.obe.util.utils.LogUtils;
import com.gx.obe.web.entity.auth.AuthUser;
import com.gx.obe.web.entity.evaluation.EvaluationStep;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.expert.ExpertEvaluationStepService;
import com.swtdesigner.ResourceManager;
import com.swtdesigner.SWTResourceManager;

/**
 * @Description: 单一的评审结果界面
 * @author chenxw
 */
public abstract class EvaluationResultSingleComposite extends Composite {
	
	private final ExpertEvaluationStepService expertEvaluationStepService = new ExpertEvaluationStepService();
	
	private Label messageLabel;
	private ELink applyReEvaluation_link;
	private Composite resultComposite;
	
	private final AuthUser curUser = Constants.USER;
	
	protected TenderProjectEntity tenderProject;
	protected EvaluationStep evaluationStep;
	protected String userId;
	
	{
		LogUtils.logClass(EvaluationResultSingleComposite.class);
	}
	
	/**
	 * @author chenxw
	 * @param parent
	 * @param style
	 */
	public EvaluationResultSingleComposite(Composite parent, int style) {
		super(parent, style);
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.verticalSpacing = 0;
		gridLayout.marginHeight = 1;
		gridLayout.marginWidth = 1;
		setLayout(gridLayout);
		
		Composite composite_1 = new Composite(this, SWT.NONE);
		composite_1.setBackgroundMode(SWT.INHERIT_FORCE);
		composite_1.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
		GridLayout gl_composite_1 = new GridLayout(2, false);
		composite_1.setLayout(gl_composite_1);
		
		messageLabel = new Label(composite_1, SWT.WRAP);
		messageLabel.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
		messageLabel.setFont(SWTResourceManager.getFont("微软雅黑", 10, SWT.NORMAL));
		messageLabel.setForeground(ColorConstants.NOTE_TIPS_COLOR);
		
		applyReEvaluation_link = new ELink(composite_1, SWT.NONE);
		applyReEvaluation_link.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				applyReEvaluationAction();
			}
		});
		applyReEvaluation_link.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1));
		applyReEvaluation_link.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/hand.png"));
		applyReEvaluation_link.setText(Messages.Button_appllyForBack);
		
		resultComposite = new Composite(this, SWT.NONE);
		resultComposite.setLayout(new FillLayout(SWT.HORIZONTAL));
		resultComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		resultComposite.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		
		createResultComposite(resultComposite);
	}
	
	/**
	 * @Description: 生成结果界面
	 * @author chenxw
	 * @param tableComposite
	 */
	protected abstract void createResultComposite(Composite tableComposite);
	
	/**
	 * @Description: 刷新结果界面
	 * @author chenxw
	 */
	protected abstract void refreshResultComposite();
	
	/**
	 * @Description: 刷新
	 * @author chenxw
	 * @param tenderProject
	 * @param evaluationStep
	 */
	public void refresh(TenderProjectEntity tenderProject, EvaluationStep evaluationStep, String userId) {
		this.tenderProject = tenderProject;
		this.evaluationStep = evaluationStep;
		this.userId = userId;
		
		refreshMessageLabel();
		refreshApplyReEvaluationLink();
		refreshResultComposite();
	}
	
	/**
	 * @Description: 刷新申请回退评审步骤按钮
	 * @author chenxw
	 */
	private void refreshApplyReEvaluationLink() {
		WidgetUtils.setControlVisible(applyReEvaluation_link, false);
		applyReEvaluation_link.getParent().layout();
		if (!CommonEnum.COMPLETE.equals(evaluationStep.getStepStatus())) {
			return;
		}
		if (!curUser.getId().equals(userId)) {
			return;
		}
		WidgetUtils.setControlVisible(applyReEvaluation_link, true);
		applyReEvaluation_link.setText(MessageFormat.format(Messages.Button_appllyForBackFormat, evaluationStep.getStepName()));
		applyReEvaluation_link.getParent().layout();
	}
	
	/**
	 * @Description: 刷新消息标签
	 * @author chenxw
	 */
	private void refreshMessageLabel() {
		String supplierName = com.gx.obe.components.core.I18N.getString(I18NEnum.SUPPLIERNAME, tenderProject.getTenderType());
		StringBuilder msg = new StringBuilder();
		if (CommonEnum.COMPLETE.equals(evaluationStep.getStepStatus())) {
			msg.append(MessageFormat.format(Messages.Tips_progressComplate, evaluationStep.getStepName()));
		} else if (CommonEnum.IN_PROGRESS.equals(evaluationStep.getStepStatus())) {
			msg.append(MessageFormat.format(Messages.Tips_progressIn, evaluationStep.getStepName()));
		} else {
			msg.append(MessageFormat.format(Messages.Tips_progressNotStart, evaluationStep.getStepName()));
		}
		msg.append("  ").append(Messages.Title_bidEvaluationRules).append(":");
		if (evaluationStep.getFinalistNumber() > 0) {
			msg.append(MessageFormat.format(Messages.Tips_evaluationRule_sortNoFilter, evaluationStep.getFinalistNumber(), supplierName));
		} else {
			msg.append(MessageFormat.format(Messages.Tips_evaluationRule_noFilter, supplierName));
		}
		messageLabel.setText(msg.toString());
	}
	
	/**
	 * @Description: 申请回退
	 * @author chenxw
	 */
	private void applyReEvaluationAction() {
		if (!applyReEvaluation_link.getEnabled()) {
			return;
		}
		ExpertEvaluationStep expertEvaluationStep = expertEvaluationStepService.getExpertEvaluationStep(tenderProject.getId(), evaluationStep.getId(), userId);
		if (Objects.isNull(expertEvaluationStep)) {
			return;
		}
		new ApplyReEvaluationDialog(getShell(), evaluationStep, expertEvaluationStep).open();
	}
	
	@Override
	protected void checkSubclass() {}
	
}
