package com.gx.obe.step.dialog;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

import org.eclipse.swt.SWT;
import org.eclipse.swt.core.MessageDialog;
import org.eclipse.swt.core.listener.TextHintListener;
import org.eclipse.swt.core.listener.VerifyRequiredListener;
import org.eclipse.swt.core.utils.ValidateUtils;
import org.eclipse.swt.core.widgets.EButton;
import org.eclipse.swt.core.widgets.EDialog;
import org.eclipse.swt.core.widgets.ESepator;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import com.gx.obe.ColorConstants;
import com.gx.obe.common.icons.IconsActivator;
import com.gx.obe.common.widget.button.Tips;
import com.gx.obe.components.core.enumeration.CommonEnum;
import com.gx.obe.components.core.enumeration.EvaluationStepEnum;
import com.gx.obe.message.Messages;
import com.gx.obe.util.utils.StringUtils;
import com.gx.obe.web.entity.evaluation.EvaluationFactor;
import com.gx.obe.web.entity.evaluation.EvaluationStep;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.evaluation.EvaluationStepService;
import com.swtdesigner.ResourceManager;
import com.swtdesigner.SWTResourceManager;

/**
 * @Description: 评审步骤对话框
 * @author chenxw
 */
public abstract class EvaluationStepDialog extends EDialog {
	
	private final EvaluationStepService evaluationStepService = new EvaluationStepService();
	
	protected TenderProjectEntity tenderProject;
	protected EvaluationStep evaluationStep;
	
	private Tips tips;
	
	private Composite fromComposite;
	
	private Text stepNameText;
	private Text stepMemoText;
	
	private boolean add;
	
	private EButton save_btn;
	
	private Composite gridComposite;
	
	private final List<EvaluationFactor> evaluationFactorList = new ArrayList<>();
	
	private final List<String> stepExpertUserIdList = new ArrayList<>();
	
	/**
	 * @author chenxw
	 * @param parentShell
	 */
	public EvaluationStepDialog(Shell parentShell, TenderProjectEntity tenderProject, EvaluationStep evaluationStep) {
		super(parentShell);
		this.tenderProject = tenderProject;
		this.evaluationStep = evaluationStep;
		this.add = Objects.isNull(evaluationStep);
	}
	
	public void addEvaluationFactor(EvaluationFactor evaluationFactor) {
		evaluationFactorList.add(evaluationFactor);
	}
	
	public void addStepExpertUserId(String stepExpertUserId) {
		stepExpertUserIdList.add(stepExpertUserId);
	}
	
	/**
	 * @Description: 创建对话框区域
	 * @author chenxw
	 * @param parent
	 * @return
	 */
	@Override
	protected Control createDialogArea(Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
		GridLayout gl_container = new GridLayout(1, false);
		gl_container.verticalSpacing = 0;
		gl_container.marginHeight = 0;
		gl_container.marginWidth = 0;
		container.setLayout(gl_container);
		
		Composite composite = new Composite(container, SWT.NONE);
		composite.setLayout(new GridLayout(1, false));
		composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		
		Composite composite_2 = new Composite(composite, SWT.NONE);
		GridLayout gl_composite_2 = new GridLayout(1, false);
		gl_composite_2.marginHeight = 0;
		gl_composite_2.marginWidth = 0;
		composite_2.setLayout(gl_composite_2);
		composite_2.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		tips = new Tips(composite_2, SWT.NONE);
		tips.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		tips.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, EvaluationStepEnum.getEvaluationStepIco(EvaluationStepEnum.DETAIL)));
		tips.setSurroundColor(SWTResourceManager.getColor(60, 138, 226));
		
		Composite composite_3 = new Composite(composite, SWT.NONE);
		composite_3.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		GridLayout gl_composite_3 = new GridLayout(2, false);
		gl_composite_3.horizontalSpacing = 0;
		gl_composite_3.marginHeight = 0;
		gl_composite_3.marginWidth = 0;
		composite_3.setLayout(gl_composite_3);
		
		fromComposite = new Composite(composite_3, SWT.NONE);
		fromComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		fromComposite.setLayout(new GridLayout(2, false));
		
		gridComposite = new Composite(composite_3, SWT.NONE);
		gridComposite.setLayout(new FillLayout(SWT.HORIZONTAL));
		gridComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, true, 1, 1));
		
		Label step_name = new Label(fromComposite, SWT.NONE);
		step_name.setFont(SWTResourceManager.getFont("微软雅黑", 9, SWT.NORMAL));
		step_name.setAlignment(SWT.RIGHT);
		step_name.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		step_name.setText(Messages.StepName + ": ");
		
		Composite stepNameComposite = new Composite(fromComposite, SWT.NONE);
		stepNameComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
		GridLayout gl_stepNameComposite = new GridLayout(1, false);
		gl_stepNameComposite.marginWidth = 0;
		gl_stepNameComposite.verticalSpacing = 1;
		gl_stepNameComposite.marginHeight = 0;
		stepNameComposite.setLayout(gl_stepNameComposite);
		
		stepNameText = new Text(stepNameComposite, SWT.NONE);
		stepNameText.setFont(SWTResourceManager.getFont("微软雅黑", 9, SWT.NORMAL));
		stepNameText.setLayoutData(new GridData(SWT.FILL, SWT.BOTTOM, true, false, 1, 1));
		stepNameText.addPaintListener(new TextHintListener(Messages.Tips_EnterEvaluationStepName));
		stepNameText.addVerifyListener(new VerifyRequiredListener(stepNameText, MessageFormat.format(Messages.Tips_required, Messages.StepName)));
		stepNameText.setTextLimit(20);
		
		ESepator label = new ESepator(stepNameComposite, SWT.NONE);
		label.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));
		label.setBackground(ColorConstants.SEP_LIGHT_COLOR);
		
		createFromArea(fromComposite, gridComposite);
		
		Label label_5 = new Label(fromComposite, SWT.NONE);
		label_5.setFont(SWTResourceManager.getFont("微软雅黑", 9, SWT.NORMAL));
		label_5.setLayoutData(new GridData(SWT.RIGHT, SWT.TOP, false, true, 1, 1));
		label_5.setText(Messages.Note + ": ");
		
		stepMemoText = new Text(fromComposite, SWT.BORDER | SWT.WRAP | SWT.V_SCROLL);
		stepMemoText.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		stepMemoText.setTextLimit(100);
		
		ESepator but_sep = new ESepator(container, SWT.SEPARATOR | SWT.HORIZONTAL);
		but_sep.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		but_sep.setBackground(ColorConstants.SEP_LIGHT_COLOR);
		
		Composite composite_1 = new Composite(container, SWT.NONE);
		composite_1.setLayout(new GridLayout(2, false));
		composite_1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		save_btn = new EButton(composite_1, SWT.NONE);
		save_btn.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/save.png"));
		save_btn.addMouseListener(MouseListener.mouseUpAdapter(e -> saveAction()));
		save_btn.setMinWidth(90);
		save_btn.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, true, false, 1, 1));
		save_btn.setText(Messages.Button_save);
		
		EButton cancel_btn = new EButton(composite_1, SWT.NONE);
		cancel_btn.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/close.png"));
		cancel_btn.addMouseListener(MouseListener.mouseUpAdapter(e -> close()));
		cancel_btn.setMinWidth(90);
		cancel_btn.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		cancel_btn.setText(Messages.Button_close);
		
		init();
		return container;
	}
	
	/**
	 * @Description: 获取提示文本
	 * @author chenxw
	 * @return
	 */
	protected abstract String getTipsText();
	
	/**
	 * @Description: 获取步骤名称默认值
	 * @author chenxw
	 * @return
	 */
	protected abstract String getStepNameDef();
	
	/**
	 * @Description: 获取步骤类型
	 * @author chenxw
	 * @return
	 */
	protected abstract String getStepType();
	
	/**
	 * @Description: 创建表单区域
	 * @author chenxw
	 * @param fromComposite
	 */
	protected abstract void createFromArea(Composite fromComposite, Composite gridComposite);
	
	/**
	 * @Description: 设置表单数据
	 * @author chenxw
	 * @param evaluationStep
	 */
	protected abstract void setFromData(EvaluationStep evaluationStep);
	
	/**
	 * @Description: 验证表单数据
	 * @author chenxw
	 * @param evaluationStep
	 */
	protected abstract String valFromData();
	
	/**
	 * @Description: 获取表单数据
	 * @author chenxw
	 * @param evaluationStep
	 */
	protected abstract void getFromData(EvaluationStep evaluationStep);
	
	/**
	 * @Description: 初始化
	 * @author chenxw
	 */
	private void init() {
		tips.setText(getTipsText());
		if (add) {
			this.setTitle(MessageFormat.format(Messages.Title_create, Messages.Step_step));
			stepNameText.setText(getStepNameDef());
		} else {
			this.setTitle(MessageFormat.format(Messages.Title_edit, Messages.Step_step));
			stepNameText.setText(Optional.of(evaluationStep).map(EvaluationStep::getStepName).orElse(getStepNameDef()));
			stepMemoText.setText(Optional.of(evaluationStep).map(EvaluationStep::getStepMemo).orElse(""));
			if (CommonEnum.COMPLETE.equals(Optional.ofNullable(evaluationStep).map(EvaluationStep::getStepStatus).orElse(null))) {
				save_btn.setEnabled(false);
			}
			setFromData(evaluationStep);
		}
	}
	
	/**
	 * @Description: 保存事件
	 * @author chenxw
	 */
	private void saveAction() {
		if (!save_btn.getEnabled()) return;
		if (!ValidateUtils.checkInput(fromComposite)) {
			return;
		}
		if (!add) {
			if (evaluationStepService.isInEvaluationStep(tenderProject.getId(), evaluationStep.getId())) {
				MessageDialog.openWarning(getShell(), Messages.Tips_startedStepEvaluation);
				return;
			}
		}
		if (evaluationStepService.valEvaluationStepNameExist(tenderProject.getId(), add ? null : evaluationStep.getId(), stepNameText.getText().trim())) {
			MessageDialog.openWarning(getShell(), Messages.Step_step + "[" + stepNameText.getText().trim() + "]" + Messages.Tips_AlreadyExistsPleaseRetype);
			stepNameText.setFocus();
			stepNameText.setText("");
			return;
		}
		
		String error = valFromData();
		if (StringUtils.isNotEmpty(error)) {
			MessageDialog.openWarning(getShell(), error);
			return;
		}
		
		evaluationStep = initEvaluationStep();
		evaluationStep.setStepName(stepNameText.getText().trim());
		evaluationStep.setStepMemo(stepMemoText.getText().trim());
		
		getFromData(evaluationStep);
		
		EvaluationStep saveStep = evaluationStepService.saveEvaluationStepAndFactors(evaluationStep, evaluationFactorList, null, stepExpertUserIdList);
		if (Objects.isNull(saveStep)) {
			MessageDialog.openError(getShell(), stepNameText.getText() + Messages.Tips_saveFail);
			return;
		}
		
		getCallBack().callBackObject(saveStep);
		
		close();
	}
	
	/**
	 * @Description: 初始化评审步骤
	 * @author chenxw
	 * @return
	 */
	private EvaluationStep initEvaluationStep() {
		if (isAdd()) {
			evaluationStep = new EvaluationStep();
			evaluationStep.setStepType(getStepType());
			evaluationStep.setTenderId(tenderProject.getId());
			evaluationStep.setStepStatus(CommonEnum.NOT_START);
			evaluationStep.setFilterType(EvaluationStepEnum.FILTER_NO);
		}
		return evaluationStep;
	}
	protected void setStepNameText(String text) {
		stepNameText.setText(text);
	}
	/**
	 * @Description: 是否新增
	 * @author chenxw
	 * @return
	 */
	public boolean isAdd() {
		return add;
	}
	
	@Override
	protected Point getInitialSize() {
		return super.getInitialSize();
	}
	
}
